/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Shell;

import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.dialog.LogbackPreferencesDialog;
import ch.qos.logback.eclipse.model.LoggingEventManager;
import ch.qos.logback.eclipse.views.LogbackView;

/**
 * This action prompts the user for a string to setup the PatternLayout
 * 
 * @author S&eacute;bastien Pennec
 */
public class LogbackPreferencesAction extends Action {

  private final Shell shell;
  private final LogbackView view;

  public LogbackPreferencesAction(Viewer viewer, LogbackView view) {
      super(Messages.LogbackPreferencesActionName);
    shell = viewer.getControl().getShell();
    this.view = view;
  }

  public void run() {
    FontData[] fd = view.getFont().getFontData();
    String name = fd[0].getName();
    int size = fd[0].getHeight();
    
    LogbackPreferencesDialog dialog = new LogbackPreferencesDialog(shell, view.getPattern(), 
        view.getServerPort(), name, size);
    if (dialog.open() != InputDialog.OK) {
      return;
    }

    if (!dialog.getPattern().equals(view.getPattern())) {
      view.updatePattern(dialog.getPattern());
    }

    if (dialog.getServerPort() != view.getServerPort()) {
      view.updateServerPort(dialog.getServerPort());
    }
    
    if (fontChanged(fd, dialog)) {
      view.updateFont(dialog.getFontName(), dialog.getFontSize());
    }
    
    if (dialog.getListMaxSize() != LoggingEventManager.getManager().getMaxSize()) {
      LoggingEventManager.getManager().setMaxSize(dialog.getListMaxSize());
    }
  }
  
  private boolean fontChanged(FontData[] fd, LogbackPreferencesDialog dialog) {
    String name = fd[0].getName();
    int size = fd[0].getHeight();
    
    if (dialog.getFontSize() != size || !dialog.getFontName().equals(name)) {
      return true;
    } else {
      return false;
    }
  }
}
