/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.views;

import org.eclipse.jface.viewers.ILazyContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;

import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.eclipse.model.LoggingEventManager;
import ch.qos.logback.eclipse.model.LoggingEventManagerEvent;
import ch.qos.logback.eclipse.model.LoggingEventManagerListener;

/**
 * The content provider class is responsible for providing objects to the view.
 * 
 * @author S&eacute;bastien Pennec
 */
public class LoggingEventContentProvider implements LoggingEventManagerListener,
    ILazyContentProvider {

  private TableViewer viewer;
  private boolean autoScroll = true;
  private int lastCount = 0;

  public LoggingEventContentProvider(TableViewer viewer) {
    this.viewer = viewer;
  }

  public void loggingEventsChanged(final LoggingEventManagerEvent event) {
    // If this is the UI thread, then make the change
    if (Display.getCurrent() != null) {
      updateViewer(event);
      return;
    }

    // otherwise redirect to execute on the UI thread
    Display.getDefault().asyncExec(new Runnable() {
      public void run() {
        updateViewer(event);
      }
    });
  }

  private void updateViewer(final LoggingEventManagerEvent event) {
    int count = LoggingEventManager.getManager().getEventCount();
    viewer.setItemCount(count);
    if (count < lastCount) {
      //we've stripped down the list, we
      //need to refresh the whole viewer
      viewer.refresh();
    }
    lastCount = count;
    if (event.getItemsAdded().length > 0 && autoScroll) {
      viewer.getTable().showItem(viewer.getTable().getItem(count - 1));
    }
  }

  public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
    this.viewer = (TableViewer) viewer;
  }

  public void toggleAutoScroll() {
    autoScroll = !autoScroll;
    int size = LoggingEventManager.getManager().getEventCount();
    if (autoScroll && size > 0) {
      viewer.getTable().showItem(viewer.getTable().getItem(size - 1));
    }
  }

  public boolean getAutoScroll() {
    return autoScroll;
  }

  public void dispose() {
    // do nothing
  }

  public void updateElement(int index) {
    LoggingEvent event = LoggingEventManager.getManager().getEvent(index); 
    if (event != null) {
      viewer.replace(event, index);
    }
  }
}
